#!/usr/bin/perl -w
use strict;

my %symbol=();
my %diso_ref=();

sub read_symbol{
    
    my $cfg_nmr="symbol.config";
    open(INPUT,$cfg_nmr) or die " $cfg_nmr : $!\n";
    while(<INPUT>){
	next if /^\s*$/ || /\#/;
	chomp;
	my ($short,$isotope) = split;
	$symbol{$short}=$isotope;
	print " Config $short Isotope = $isotope \n";
    } 
    close(INPUT);
}

sub read_ref{
    
    my $cfg_nmr="nmr_ref.config";
    open(INPUT,$cfg_nmr) or die " $cfg_nmr : $!\n";
    while(<INPUT>){
	next if /^\s*$/ || /\#/;
	chomp;
	my ($short,$ref) = split;
	$diso_ref{$short}=1.0*$ref;
	print " Config sigma_ref $short = $ref ppm \n";
    } 
    close(INPUT);
}

#-------------------------------------

if ($#ARGV<0){
    print " Usage $0 file (file.scf file.nmr file.efg ) \n";
    exit(1);
}

my $ifile="$ARGV[0].scf";
my $odir="$ARGV[0]";
print "------------------------------------\n";
read_symbol();
print "------------------------------------\n";
read_ref();
print "------------------------------------\n";
print " Processing file $ifile \n";

if ( ! -d $odir ) {
    print " Create dir $odir \n";
    mkdir $odir;
}

my %hash=();
my $atom="XX";
my $Cq="";
my $diso="";
my $aniso="";
my $eta="";
my @lattice=();
my $atomic_specy="";

open(SCF,"<$odir.scf")
      or die " Can't open file : $!\n";
while(<SCF>){
    if ($_=~/crystal axes/){
	for(my $i=0;$i<3;$i++){
	    $_=<SCF>;
	    chomp;
	    my @var = split;
	    for(my $j=3;$j<6;$j++){
		push @lattice, $var[$j];
	    }
	}
    }
}
close (SCF);

print "------------------------------------\n";
print " LATTICE PARAMETERS \n";
print  "------------------------------------\n";
for(my $i=0;$i<3;$i++){
    print " Lattice ",$lattice[$i],",",$lattice[$i+3],",",$lattice[$i+6],"\n";
}

$ifile="$ARGV[0].nmr";
print "------------------------------------\n";
print " Processing file $ifile \n";
open(NMR,"<$ifile")
    or die " Can't open file : $!\n";
my $count_atom=0;
while(<NMR>){

    if ($_=~/Total NMR chemical shifts in ppm/){

	while(<NMR>){

	    if ($_=~/Atom/){
		chomp;
		my @var= split;

		my $symbol=$var[2];
		my $num=$var[1];
		my $atom="$symbol$num";
		my $atomic_specy=$symbol;
		my @lab= ("atom",$symbol,$num);
		push @{$hash{$atom}}, \@lab;

		$var[7] =~ s/[\(\)]//g;

		my @pos= ("position",$var[5],$var[6],$var[7]);
		push @{$hash{$atom}}, \@pos;

		my $diso= - ( $var[9] - $diso_ref{$atomic_specy}) ;
		my @dcs= ("shielding","$diso","0.0","0.0");
		push @{$hash{$atom}}, \@dcs;
		
		print " $atom\n";
		my @CSA=("CSA");
		for(my $i=0;$i<3;$i++){
		    $_=<NMR>;
		    chomp;
		    print " $_\n";
		    my @lvar = split;
		    for(my $j=0;$j<3;$j++){
			push @CSA, $lvar[$j];
		    }
		}
		push @{$hash{$atom}}, \@CSA; 
		$count_atom+=1;
	    }
	}
    }
}
close(NMR);
print "------------------------------------\n";
print " Number of atoms : $count_atom \n";
$ifile="$ARGV[0].efg";
print "------------------------------------\n";
print " Processing file $ifile \n";
open(EFG,"<$ifile")
    or die " Can't open file : $!\n";
while(<EFG>){

    if ($_=~/Total EFG calculation:/){
	$_=<EFG>;
	for(my $n=0;$n<$count_atom;$n++){

	    print "-------------\n";

	    my $symbol="";
	    my $num="";
	    my $atom="";

	    my @EFG=("EFG");
	    for(my $i=0;$i<3;$i++){
		$_=<EFG>;
		chomp;
		my @var = split;
		
		$symbol=$var[0];
		$num=$var[1];
		$atom="$symbol$num";
		print " $_\n";
		for(my $j=3;$j<6;$j++){
		    push @EFG, $var[$j];
		}
	    }
	    print " \tEFG $atom done\n";
	    push @{$hash{$atom}}, \@EFG; 
	    
	    $_=<EFG>;
	    $_=<EFG>;
	    $_=<EFG>;
	    $_=<EFG>;
	    chomp;
	    my @lvar = split;
	    my $Cq=abs($lvar[7]);
	    my $nq=abs($lvar[10]);
	    my @efg= ("quadrupolar",$Cq,$nq);
	    push @{$hash{$atom}}, \@efg;
	    $_=<EFG>;
	    $_=<EFG>;
	}
    }
}
close(EFG);

my $data="$odir/input";
open(OUTPUT,">$data")
    or die " Output $data File cannot be opened \n";

for(my $i=0;$i<3;$i++){
    print OUTPUT "Lattice ",$lattice[$i],",",$lattice[$i+3],",",$lattice[$i+6],"\n";
}

my $count=0;
my $type="";
my $coord=0;
foreach my $atom (sort {
    ${${$hash{$a}}[0]}[1] cmp ${${$hash{$b}}[0]}[1] ||
    ${${$hash{$a}}[0]}[2] <=> ${${$hash{$b}}[0]}[2] ;
} keys %hash){
    print "\n Atom : $atom \n";
    $count+=1;
    $coord=0;
    $diso="";
    for my $val (@{$hash{$atom}}){
	my $arg=${$val}[0];
	shift @$val;
	if ("$arg" eq "atom"){
	    my ($nucleus,$id) = @{$val};
	    print "\tnucleus = $nucleus id = $id\n";
	    if ( $nucleus ne $type){
		$type=$nucleus;
		print OUTPUT "Newtype ${symbol{$nucleus}} \n";
	    }
	}
	elsif( "$arg" eq "position"){
	    if ($coord==0){
		my ($x,$y,$z) = @{$val};
		print "\tposition $x $y $z\n";
		print OUTPUT "Coord ",$x,",",$y,",",$z,"\n";
		$coord=1;
	    }
	}
	elsif( "$arg" eq "quadrupolar"){
	    my ($Cq,$nq) = @{$val};
	    print "\tEFG Cq = $Cq Eta = $nq \n";
	    print OUTPUT "NMR  ",$Cq,",",$nq,",",$diso,"\n";
	}
	elsif( "$arg" eq "shielding"){
	    my ($iso,$ani,$eta) = @{$val};
	    $diso=$iso;
	    print "\tCS diso = $iso Anisotropy = $ani Asymmetry = $eta\n";
	}
	elsif( "$arg" eq "CSA"){
	    my @csa=@$val;
	    print "\n\tChemical Shielding Tensor \n";
	    for(my $i=0;$i<9;$i+=3){
		print "\tCS  ",$csa[$i]," ",$csa[$i+1]," ",$csa[$i+2],"\n";
	    }
	    for(my $i=0;$i<3;$i++){
		print OUTPUT "CSA ",$csa[$i],",",$csa[$i+3],",",$csa[$i+6],"\n";
	    }
	}
	elsif( "$arg" eq "EFG"){
	    my @efg=@$val;
	    print "\n\tElectric Field Gradient Tensor \n";
	    for(my $i=0;$i<9;$i+=3){
		print "\tEFG ",$efg[$i]," ",$efg[$i+1]," ",$efg[$i+2],"\n";
	    }
	    for(my $i=0;$i<3;$i++){
		print OUTPUT "EFG ",$efg[$i],",",$efg[$i+3],",",$efg[$i+6],"\n";
	    }
	}
    }
}

close(OUTPUT);
print " Number of atoms $count \n";
exit(0);
